/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <efl_extension.h>
#include "$(appName).h"
#include "data.h"
#include "view.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *gengrid;
	Elm_Gengrid_Item_Class *itc;
	Eina_List *app_list;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.gengrid = NULL,
	.itc = NULL,
	.app_list = NULL,
};

#define BUF_SIZE 1024
#define POPUP_TIMEOUT 5.0
#define GENGRID_ITEM_WIDTH 300
#define GENGRID_ITEM_HEIGHT 300

static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _win_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _block_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _gengrid_item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *_get_grid_content_cb(void *data, Evas_Object *obj, const char *part);
static void _create_popup(Evas_Object *parent, app_info_h *app_info);
static void _initialize_gengrid_item_class(void);
static Evas_Object *_create_gengrid(Evas_Object *parent);
static Elm_Object_Item *_append_item(Evas_Object *gengrid, app_info_h *app_info);

/**
 * @brief Creates essential objects: window, conformant and layout.
 */
Eina_Bool view_create(void *user_data)
{
	Eina_List *it = NULL;
	app_info_h *app_info = NULL;

	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/* Create the conformant */
	s_info.conform = view_create_conformant_without_indicator(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	s_info.gengrid = _create_gengrid(s_info.win);
	s_info.app_list = get_all_apps();
	EINA_LIST_FOREACH(s_info.app_list, it, app_info)
		_append_item(s_info.gengrid, app_info);

	eext_object_event_callback_add(s_info.gengrid, EEXT_CALLBACK_BACK, _win_back_cb, NULL);

	elm_object_part_content_set(s_info.conform, "elm.swallow.content", s_info.gengrid);

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);
	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_autodel_set(win, EINA_TRUE);
	elm_win_indicator_mode_set(win, ELM_WIN_INDICATOR_SHOW);

	evas_object_smart_callback_add(win, "delete,request", _delete_win_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a conformant without indicator for wearable app.
 * @param[in] win The object to which you want to set this conformant
 * Conformant is mandatory for base GUI to have proper size
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	destroy_all_apps_list(s_info.app_list);

	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	/*
	 * Write your code here for smart callback.
	 */
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_win_lower(s_info.win);
}

/**
 * @brief Internal callback function invoked on popup window content click.
 * This function is responsible for popup window removal.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _block_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

/**
 * @brief Internal callback function invoked on application's icon click.
 * This function is responsible for displaying a popup window containing selected application's information.
 * @param[in] data The user data passed to the elm_gengrid_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _gengrid_item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	_create_popup(obj, (app_info_h *)data);
}

/**
 * @brief Internal callback function invoked on gengrid's item display.
 * This function is assigned to the func.content_get handler of gengrid item's class structure.
 * The item's area consists of a swallow where the application's icon is set:
 * @param[in] data The user data passed to the elm_gengrid_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The object to be displayed is returned.
 */
static Evas_Object *_get_grid_content_cb(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *image;
	Evas_Object *bg;
	char *icon;
	app_info_h *app_info = (app_info_h *)data;

	if (!strcmp(part, "elm.swallow.icon")) {
		image = elm_image_add(obj);
		app_info_get_icon(*app_info, &icon);
		if (!elm_image_file_set(image, icon, NULL)) {
			evas_object_del(image);
			bg = elm_bg_add(obj);
			elm_bg_color_set(bg, 255, 255, 0);
			return bg;
		}

		return image;
	}

	return NULL;
}

/**
 * @brief Internal function which creates a popup window for application related
 * information display.
 * This function is invoked once the application icon is tapped.
 * @param[in] parent The popup window's parent.
 * @param[in] app_info The handle of the application's information structure,
 * which details are to be displayed.
 */
static void _create_popup(Evas_Object *parent, app_info_h *app_info)
{
	char *label;
	char *icon;
	char *package;
	char info[BUF_SIZE];
	Evas_Object *popup = elm_popup_add(parent);

	get_app_label(app_info, &label);
	get_app_icon(app_info, &icon);
	get_app_package(app_info, &package);

	elm_object_part_text_set(popup, "title,text", label);

	snprintf(info, BUF_SIZE, "Icon path:<br>%s<br><br>Package:<br>%s", icon, package);
	elm_object_part_text_set(popup, "default", info);

	free(label);
	free(icon);
	free(package);

	evas_object_smart_callback_add(popup, "block,clicked", _block_clicked_cb, NULL);
	elm_popup_timeout_set(popup, POPUP_TIMEOUT);

	evas_object_show(popup);
}

/**
 * @brief Internal function which creates a gengrid widget for application icons display.
 * @param[in] parent The gengrid object parent.
 * @return Function returns gengrid object if it was successfully created,
 * otherwise NULL is returned.
 */
static Evas_Object *_create_gengrid(Evas_Object *parent)
{
	Evas_Object *gengrid;

	if (!s_info.itc)
		_initialize_gengrid_item_class();

	gengrid = elm_gengrid_add(parent);
	elm_gengrid_item_size_set(gengrid, GENGRID_ITEM_WIDTH, GENGRID_ITEM_HEIGHT);
	elm_gengrid_reorder_mode_set(gengrid, EINA_TRUE);
	elm_gengrid_horizontal_set(gengrid, EINA_TRUE);
	evas_object_size_hint_weight_set(gengrid, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(gengrid, EVAS_HINT_FILL, EVAS_HINT_FILL);

	return gengrid;
}

/**
 * @brief Internal function which appends an application's icon to the gengrid widget.
 * @param[in] gengrid The target container for icon display.
 * @param[in] app_info The information handle of the application which icon is to be appended.
 * @return Function returns gengrid object containing the icon if it was successfully appended,
 * otherwise NULL is returned.
 */
static Elm_Object_Item *_append_item(Evas_Object *gengrid, app_info_h *app_info)
{
	Elm_Object_Item *item;

	if (!s_info.itc)
		_initialize_gengrid_item_class();

	item = elm_gengrid_item_append(gengrid, s_info.itc, app_info, _gengrid_item_selected_cb, app_info);
	return item;
}

/**
 * @brief Creates an item class for gengrid object.
 */
static void _initialize_gengrid_item_class(void)
{
	s_info.itc = elm_gengrid_item_class_new();
	s_info.itc->item_style = "default";
	s_info.itc->func.text_get = NULL;
	s_info.itc->func.content_get = _get_grid_content_cb;
	s_info.itc->func.state_get = NULL;
	s_info.itc->func.del = NULL;
}
